#include "mbed.h"
#include "string.h"
#include "iostream"
#include "stdio.h"
#include <stdlib.h>

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables des fonctions d'interruptions liées au bouton poussoir de la carte nucléo

//Bouton poussoir de la carte nucléo : sert pour tester une fonction en appuyant sur le bouton
InterruptIn mon_int(PC_13);
bool RG_etat=0; //Initialisation de l'interrupteur

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables et en-têtes des fonctions de déplacement de base du robot

//Moteur1 : configuration des entrées
PwmOut mot1_hor(PA_10);
PwmOut mot1_trig(PA_15);
InterruptIn front_captmot1(PB_0); // Détection front montant moteur 1

//Moteur2 : configuration des entrées
PwmOut mot2_hor(PB_14);
PwmOut mot2_trig(PB_13);
InterruptIn front_captmot2(PB_2); // Détection front montant moteur 2

//Moteur3 : configuration des entrées
PwmOut mot3_trig(PC_9);
PwmOut mot3_hor(PB_8);
InterruptIn front_captmot3(PB_4); // Détection front montant moteur 3

//rapports cycliques attribués à chacun des moteurs. Ils définissent la vitesse de rotation de la roue.
double RG_Consigne_rc_1=0;
double RG_Consigne_rc_2=0;
double RG_Consigne_rc_3=0;

//Variables définissant le sens de rotation de chaque roue. +1 -> rotation dans le sens trigonométrique, -1 -> rotation dans le sens horaire 
int RG_Sens_rota1=1;
int RG_Sens_rota2=1;
int RG_Sens_rota3=1;

//Commandes de base des moteurs

//Fonctions pour faire tourner chaque roue indépendament avec le rapport cyclique RG_Consigne_rc_i et dans le sens RG_Sens_rotai (i=1,2,3)
void fun_RG_mot1();
void fun_RG_mot2();
void fun_RG_mot3();

//Fonctions permettant d'arrêter chaque roue indépendemment
void fun_RG_stop_mot1();
void fun_RG_stop_mot2();
void fun_RG_stop_mot3();

//Fonction permettant d'actionner les 3 moteurs avec les rapports cycliques et sens arbitrairement choisis en appuyant sur le bouton de la carte nucléo
void fonction_interruption1_test_roues();

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables et en-têtes des fonctions de déplacement avancées du robot

double RG_rc_avance=0.2; //rapport cyclique utilisé lorsque le robot avance
double RG_rc_rota=0.1; //rapport cyclique utilisé lorsque le robot tourne sur lui-même


//Fonctions de controle du moteur 3 permettant au robot de s'arreter après une instruction

int N_creneau3=0; //Nombre de crenéaux détéctés par le capteur du moteur 3 (capteur incrémental). Garde en mémoire le déplacement de la roue
void itere_N_capt3(void); //Itère la variable N_creneau3 quand le capteur du moteur 3 détecte un incrément. 

//Fonction permettant au robot de stopper après avoir effectué une rotation de 90°. Le controle s'effectue sur une roue uniquement
void stop_90_mot3 (void);

//Fonctions permettant au robot de stopper après avoir avancé de 30cm. Le controle s'effectue sur une roue uniquement
void stop_case();

Ticker mesure_roue3_ticker; // Ticker appelant régulièrement les fonctions stop_case et stop_90_mot3. Permet au robot de s'arreter lors de ces fonctions


//Fonctions de déplacement avancées du robot

void fun_RG_avancertoutdroit(void); //Fait avancer le robot de 30cm en actionnant les moteurs 2 et 3
void fun_RG_tourne_90_trig(void);//Fait tourner le robot sur lui-même de 90° dans le sens trigonométrique en actionnant les 3 moteurs
void fun_RG_tourne_90_hor(void);//Fait tourner le robot sur lui-même de 90° dans le sens horaire en actionnant les 3 moteurs

//Fonctions d'interruptions permettant de tester les 3 fonctions décrites ci-dessus en appuyant sur le bouton poussoir de la carte nucléo

void fonction_interruption_avance(void); //Permet de tester la fonction fun_RG_avancertoutdroit
void fonction_interruption_tourne_90_trig(void);//fun_RG_tourne_90_trig
void fonction_interruption_tourne_90_hor();// Permet de tester la fonction fun_RG_tourne_90_hor

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables et en-têtes des fonctions d'interprétation d'une instruction

//liste d'intruction donnee de taille très grande pour ne pas avoir changer la taille selon l'instruction donnée
 #define Taille_instructions 100
 char C_Destination[3]={'F','6'}; //Envoyé par Bluetooth, contient la case que doit atteindre le robot 

 void fun_RG_execut(void);  //fonction qui recupere les infos et les execute

 void RG_test_Ready(void); //Fonction testant si le robot peut passer à l'instruction suivante
 bool RG_isReady=1; //Indique si le robot a fini l'instruction en cours et est prêt à éxecuter la suivante
 Ticker check_Ready;//Ticker qui permet d'appeler régulièrement la fonction RG_test_Ready

 void RG_decrypte_instruction (void);  //fonction qui interprète chaque instruction individuelle
 char RG_ordre; //variable qui stocke l'instruction à traiter
 int RG_i=0; //variable parcourant le tableau d'instruction. On exécute la RG_i.ème instruction du tableau



//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables et en-têtes des fonctions quadrillage

void fun_m_get_case_vide_adjacente(int position[2], int vide[2]);
void fun_m_calcul_trajectoire(int position_init[2]);
void fun_m_calcul_suite_de_pos(int position_actu[2], int vide_fin[]);

void fun_m_finding_box(char box_name[2], int coord_box[2]);

void fun_m_remplir_liste_actions(int pos_init[2]);
void fun_m_detect_deplacement_case_suiv(int pos_actu[2], int pos_suiv[2], char m_Deplacements1case[3][1]);

//Variables Marie

int m_Nombre_deplacement = 0; // Nombre de déplacements nécessaires pour effectuer le trajet
int m_Dir = 1; // Direction du robot (WEST) W-1 N-2 E-3 S-4
int m_Nb_actions_tot = 0;
int m_Nb_actions_case = 0;
char m_Liste_actions[Taille_instructions][1];
int m_Matrice_quadrillage[6][7] = { // matrice[no ligne][no col]
                                {1, 2, 3, 2, 1, 0, 4},  //0
                                {0, 0, 0, 0, 0, 0, 6},  //1
                                {1, 2, 2, 3, 3, 0, 4},  //2
                                {1, 2, 3, 2, 3, 0, 5},  //3
                                {0, 0, 0, 0, 0, 0, 6},  //4
                                {1, 2, 3, 2, 1, 0, 7}   //5
                                }; // [ligne][colonne]
//                               0  1  2  3  4  5  6

char m_Noms_matrice_quadrillage[6][7][3] = {
                                {"P1", "A1", "F1", "A6", "P5", " 0", "C1"}, // 0
                                {" 0", " 0", " 0", " 0", " 0", " 0", "L1"}, // 1
                                {"P2", "A2", "A5", "F4", "F5", " 0", "C2"}, // 2
                                {"P3", "A3", "F2", "A7", "F6", " 0", "R0"}, // 3
                                {" 0", " 0", " 0", " 0", " 0", " 0", "L2"}, // 4
                                {"P4", "A4", "F3", "A8", "P6", " 0", "F0"}}; // 5
//                                 0    1     2     3     4     5     6

int POSITION[2]={3,6};
int RG_DIR=1;
int TAB_DIR_VERTI[4]={0,-1,0,1};
int TAB_DIR_HORIZ[4]={-1,0,1,0};

int tab_liste_positions[100][2];

char test[Taille_instructions];

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Variables et en-têtes des fonctions permettant de controler la distance parcourue et de prévenir l'utilisateur en cas de batterie faible
// [EN COURS DE DÉVELOPPEMENT]


int Controle_distance1=0; //Compte la distance que parcourt la roue 1
int Controle_distance2=0; //Compte la distance que parcourt la roue 2
int Controle_distance3=0; //Compte la distance que parcourt la roue 3

Ticker controle_batterie;

 void fun_test_distance(void);

 bool BatterieFaible=0; 


//------------------------------------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Main du programme

int main() {

    
 //Initialisation a  0  
    //Moteur 1
    mot1_hor.period_ms(10);
    mot1_trig.period_ms(10);
    mot1_hor.write(0);
    mot1_trig.write(0);


    //Moteur 2
    mot2_hor.period_ms(10);
    mot2_trig.period_ms(10);
    mot2_hor.write(0);
    mot2_trig.write(0);

    //Moteur 3
    mot3_hor.period_ms(10);
    mot3_trig.period_ms(10);
    mot3_hor.write(0);
    mot3_trig.write(0);

    //Calcul de la trajectoire à parcourir à l'aide des fonctions du bloc Quadrillage
    fun_m_calcul_trajectoire(POSITION);
    fun_m_remplir_liste_actions(POSITION);

    //Execute la liste d'instruction calculée lorsqu'on appuie sur le bouton poussoir
    mon_int.rise(&fun_RG_execut);


    //Fonctions d'interruption permettant de tester les fonctions de déplacement avancées. Sert pour controler la précision de ces fonctions.

    //mon_int.rise(&fonction_interruption_avance);
    //mon_int.rise(&fonction_interruption_tourne_90_trig);
   

    //Prototype de controle de la batterie

    //controle_batterie.attach(&fun_test_distance,300000ms);


    while(1){
    }
}

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Fin du main

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Fonctions de déplacement de base du robot

//Fonctions pour faire tourner les roues

void fun_RG_mot1()
//Fait tourner le moteur 1 à une vitesse et dans un sens choisi par l'appel de la fonction
//RG_consigne_rc : double entre 0 et 1. C'est le rapport cyclique du moteur à faire tourner  -> joue sur la vitesse de rotation
//RG_sens_rota est égal à +-1, +1 donne un sens trigo de rotation de la roue, -1 un sens horaire
{
    if(RG_Sens_rota1>0){
            mot1_trig.write(RG_Consigne_rc_1);
            mot1_hor.write(0);
    }
    else {
            mot1_trig.write(0);
            mot1_hor.write(RG_Consigne_rc_1);
    }
}

void fun_RG_mot2()
//Fait tourner le moteur 2 à une vitesse et dans un sens choisi par l'appel de la fonction
//RG_consigne_rc : double entre 0 et 1. C'est le rapport cyclique du moteur à faire tourner  -> joue sur la vitesse de rotation
//RG_sens_rota est égal à +-1, +1 donne un sens trigo de rotation de la roue, -1 un sens horaire
{
    if(RG_Sens_rota2>0){
            mot2_trig.write(RG_Consigne_rc_2);
            mot2_hor.write(0);
    }
    else {
            mot2_trig.write(0);
            mot2_hor.write(RG_Consigne_rc_2);
    }
}

void fun_RG_mot3()
//Fait tourner le moteur 3 à une vitesse et dans un sens choisi par l'appel de la fonction
//RG_consigne_rc : double entre 0 et 1. C'est le rapport cyclique du moteur à faire tourner  -> joue sur la vitesse de rotation
//RG_sens_rota est égal à +-1, +1 donne un sens trigo de rotation de la roue, -1 un sens horaire
{
    if(RG_Sens_rota3>0){
            mot3_trig.write(RG_Consigne_rc_3);
            mot3_hor.write(0);
    }
    else {
            mot3_trig.write(0);
            mot3_hor.write(RG_Consigne_rc_3);
    }
}

//Fonctions permettant de stopper les moteurs

void fun_RG_stop_mot1()
{
    mot1_trig.write(0);
    mot1_hor.write(0);

}

void fun_RG_stop_mot2()
{
    mot2_trig.write(0);
    mot2_hor.write(0);

}
void fun_RG_stop_mot3()
{
    mot3_trig.write(0);
    mot3_hor.write(0); 

}

void fonction_interruption1_test_roues() //Actionne les 3 roues avec les rapports cycliques et sens choisis arbitrairement
{
        
    // ACTIONS A REALISER LORS DE L'ACTIVATION DE L'INTERRUPTION 1
        
        RG_etat = 1-RG_etat; // changement d'etat allumé/étéint (détection front montant bouton)
        
        if(RG_etat == 1){
            RG_Sens_rota1=-RG_Sens_rota1;
            //Moteur 1
            fun_RG_mot1();

            //Moteur 2
            fun_RG_mot2();

            //Moteur 3
            fun_RG_mot3();
          
        }
        else{

            fun_RG_stop_mot1();

            //Moteur 2
            fun_RG_stop_mot2();

            //Moteur 3
            fun_RG_stop_mot3();
        } 
}

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Fonctions de déplacement avancées du robot

void itere_N_capt3(void){
    N_creneau3 = N_creneau3 + 1; //Compte le nombre de créneaux détéctés pour le moteur 3
} 

void stop_90_mot3() {       
    // Fonction d'interruption du ticker lié au moteur 3 qui stoppe le moteur quand le robot a tourne de 90°

    if (N_creneau3>= 23150){
        fun_RG_stop_mot3();
        fun_RG_stop_mot1();
        fun_RG_stop_mot2();
        RG_isReady=1;
    }
}

void stop_case() {       
    // Fonction d'interruption du ticker lié au moteur 3 qui stoppe le moteur quand le robot a tourne de 90°

    
    if (N_creneau3>= 39700){
        fun_RG_stop_mot3();
        fun_RG_stop_mot1();
        fun_RG_stop_mot2();
        RG_isReady=1;
    }
}

void fun_RG_avancertoutdroit()
//On actionne les roues 2 et 3 pour faire avancer le robot, de sorte à ce que la batterie soit à l'arrière
//Le robot avance d'une case, ie de 30cm 
//la vitesse de déplacement du robot est donnée par le rapport cyclique RG_rc_avance
//Compte le nombre de créneaux détéctés par le moteur 3 lors du déplacement
//Fixe RG_isReady=0 le temps que le robot se déplace

{
    RG_isReady=0;
    N_creneau3=0;
    POSITION[0]=POSITION[0]+TAB_DIR_VERTI[RG_DIR-1];
    POSITION[1]=POSITION[1]+TAB_DIR_HORIZ[RG_DIR-1];
    Controle_distance2=Controle_distance2+32.1;//les roues 2 et 3 avancent de 32 cm à chaque fois que le robot avance de 30cm
    Controle_distance3=Controle_distance3+32.1;
    mesure_roue3_ticker.attach(&stop_case,1ms);
    RG_Consigne_rc_2=RG_rc_avance;
    RG_Consigne_rc_3=RG_rc_avance;

    RG_Sens_rota2=1;
    RG_Sens_rota3=-1;

    fun_RG_mot2();
    fun_RG_mot3();
    front_captmot3.rise(&itere_N_capt3);
}

void fun_RG_tourne_90_trig()
// Fait un tour de 90° dans le sens trigonométrique 
//La vitesse de rotation est donnée par le rapport cyclique RG_rc_rota
//Compte le nombre de créneaux détéctés par le moteur 3 lors du déplacement
//Fixe RG_isReady=0 le temps que le robot effectue la rotation

{   
    RG_isReady=0;
    N_creneau3=0;
    if (RG_DIR==1)
    {RG_DIR=4;}
    else {RG_DIR=RG_DIR-1;}
    Controle_distance2=Controle_distance2+18.7;
    Controle_distance1=Controle_distance1+18.7;
    Controle_distance3=Controle_distance3+18.7;
    mesure_roue3_ticker.attach(&stop_90_mot3,1ms);

    RG_Consigne_rc_1=RG_rc_rota;
    RG_Consigne_rc_2=RG_rc_rota;
    RG_Consigne_rc_3=RG_rc_rota;

    RG_Sens_rota1=-1;
    RG_Sens_rota2=-1;
    RG_Sens_rota3=-1;

    fun_RG_mot1();
    fun_RG_mot2();
    fun_RG_mot3();

    front_captmot3.rise(&itere_N_capt3);

}
void fun_RG_tourne_90_hor()
// Fait un tour de 90° dans le sens horaire
//La vitesse de rotation est donnée par le rapport cyclique RG_rc_rota
//Compte le nombre de créneaux détéctés par le moteur 3 lors du déplacement
//Fixe RG_isReady=0 le temps que le robot effectue la rotation
{   
    RG_isReady=0;
    N_creneau3=0;
    if (RG_DIR==4)
    {RG_DIR=1;}
    else {RG_DIR=RG_DIR+1;}

    Controle_distance2=Controle_distance2+18.7;
    Controle_distance1=Controle_distance1+18.7;
    Controle_distance3=Controle_distance3+18.7;

    mesure_roue3_ticker.attach(&stop_90_mot3,1ms);
   
    RG_Consigne_rc_1=RG_rc_rota;
    RG_Consigne_rc_2=RG_rc_rota;
    RG_Consigne_rc_3=RG_rc_rota;

    RG_Sens_rota1=1;
    RG_Sens_rota2=1;
    RG_Sens_rota3=1;

    fun_RG_mot1();
    fun_RG_mot2();
    fun_RG_mot3();

    front_captmot3.rise(&itere_N_capt3);

}


void fonction_interruption_avance()
{
        
    // ACTIONS A REALISER LORS DE L'ACTIVATION DE L'INTERRUPTION 1
        
        RG_etat = 1-RG_etat; // changement d'etat allumé/étéint (détection front montant bouton)
        
        if(RG_etat == 1){

            fun_RG_avancertoutdroit();
          
        }
        else{
            fun_RG_stop_mot1();
            fun_RG_stop_mot2();
            fun_RG_stop_mot3();
        } 
}

void fonction_interruption_tourne_90_trig()
{
    RG_etat = 1-RG_etat; // changement d'etat allumé/étéint (détection front montant bouton)
    if(RG_etat == 1){

            fun_RG_tourne_90_trig();
          
        }
    else{
            fun_RG_stop_mot1();
            fun_RG_stop_mot2();
            fun_RG_stop_mot3();
        } 
}

void fonction_interruption_tourne_90_hor()
{
    RG_etat = 1-RG_etat; // changement d'etat allumé/étéint (détection front montant bouton)
    if(RG_etat == 1){

            fun_RG_tourne_90_hor();
          
        }
    else{
            fun_RG_stop_mot1();
            fun_RG_stop_mot2();
            fun_RG_stop_mot3();
        } 
}

//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Fonctions d'interprétation d'une instruction

void fun_RG_execut()
//fonction qui lit une liste d'instruction et qui l'execut
//Instructions=variable global = liste d'intructions
{
    RG_ordre=m_Liste_actions[RG_i][0]; //Récupère l'instruction qu'il faut réaliser dans la liste d'instructions
    check_Ready.attach(&RG_test_Ready,10ms); //Si l'instruction précédente et finie, on execute la nouvelle instruction
}

void RG_test_Ready(void)
{
    if (RG_isReady&&RG_i<Taille_instructions)
    {
        RG_decrypte_instruction();
    }
}

void RG_decrypte_instruction (void)
{
    if (RG_ordre=='A') //Instruction d'avancer
    {
        fun_RG_avancertoutdroit();
    }
    if (RG_ordre=='T') //Instruction tourner dans le sens trigonométrique
    {
        fun_RG_tourne_90_trig();
    }
    if (RG_ordre=='H') //Instruction tourner dans le sens horaire
    {
        fun_RG_tourne_90_hor();
    }
    if (RG_ordre=='S') //Fin de la série d'instructions
    {
        RG_isReady=0;
        RG_i=5000;
    }

    RG_i=RG_i+1;
    if(RG_i<Taille_instructions)
    {
        fun_RG_execut();
    }
    else {
        RG_isReady=0;
    }
}


//------------------------------------------------------------------------------------------------------------------------------------------------------------
//Fonctions permettant de controler la distance parcourue et de prévenir l'utilisateur en cas de batterie faible
// [EN COURS DE DÉVELOPPEMENT]

void fun_test_distance()
{
    int D_tot=Controle_distance1+Controle_distance2+Controle_distance3;
    if (D_tot>306000)
    {
        BatterieFaible=1;
    }
}

//------------------------------------------------------------------------------------------------------------------------------------------------------------

//Fonctions Quadrillage

void fun_m_get_case_vide_adjacente(int position[2], int vide[2]){
// Renvoie la position de la case vide adjacente à la case en entrée (ou égale)
// int position[2] : position de la case initiale du robot
// int vide[2] : position de la case vide correspondant à l'entrée de la case position (case adjacente)
    // Selon la position, on sait où se situe la case d'entrée (à gauche, en haut ou en bas)

    if(position[1] == 6){ // rangée de cases les plus à droite : la case d'entrée est à sa gauche
        vide[0]=position[0];
        vide[1]=5;
    }
    else{
        if(position[0]==0 || position[0]==3){ // Cases au-dessus d'une case vide
            vide[0]=position[0]+1;
            vide[1]=position[1];
        }
        else{
            if(position[0]==2 || position[0]==5){ // Cases en-dessous d'une case vide
                vide[0]=position[0]-1;
                vide[1]=position[1];
            }
            else{ // La case est deja vide
                vide[0] = position[0];
                vide[1] = position[1];
            }
        }
    }
}



void fun_m_calcul_trajectoire(int position_init[2]){
// Détermination du chemin à parcourir case par case pour aller d'une case initial à une case finale

// position_init position de départ du robot
// position_fin : position d'arrivée du robot
// tab_liste_positions : liste des positions successives à parcourir par le robot

    
    m_Nombre_deplacement = 0;
    //int nombre_deplacement_temp = 0;
    int position_fin[2];
    fun_m_finding_box(C_Destination, position_fin);
    int vide_init[2], vide_fin[2], position_actu[2]; // case d'entrée initiale, case d'entrée finale/intermédiaire et avancement du calcul
    int vide_fin_intermediaire[2]; // case de fin totale
    int besoin_fin_intermediaire = 0; // booléen indiquant qu'il y a une case intermédiaire
   // int pos_suiv1[2], pos_suiv2[2]; // 2 possibilités pour la valeur suivante de position actu
   // int delta_verti, delta_horiz; // ecart entre la position actuelle et l'arrivée

//   printf("ENTREE fun_m_calcul_trajectoire\n");

    fun_m_get_case_vide_adjacente(position_init, vide_init); // remplissage de vide_init

    if(m_Matrice_quadrillage[position_init[0]][position_init[1]] != 0){
        // si la case de départ n'est pas vide, le premier déplacement consiste à se placer sur la case vide adjacente
        tab_liste_positions[m_Nombre_deplacement][0] = vide_init[0];
        tab_liste_positions[m_Nombre_deplacement][1] = vide_init[1];
        m_Nombre_deplacement ++;
    }
    // premier déplacement
//        tab_liste_positions[m_Nombre_deplacement][0] = vide_init[0];
//        tab_liste_positions[m_Nombre_deplacement][1] = vide_init[1];
    fun_m_get_case_vide_adjacente(position_fin, vide_fin);  // remplissage de vide_fin_final

    // la position "actuelle" virtuelle du robot sur le trajet
    position_actu[0] = vide_init[0];
    position_actu[1] = vide_init[1];

    //printf("Pos actuu : %d, %d\n\n", position_actu[0], position_actu[1]);
    //printf("%d \n", m_Nombre_deplacement);
    //memcpy(position_actu, vide_init, sizeof position_actu); // copie vide_init dans position_actu

    vide_fin_intermediaire[0] = vide_fin[0];
    vide_fin_intermediaire[1] = vide_fin[1];
    // condition sur la position relative des cases initialles et finales du robot :
    // Besoin de définir une case finale "intermédiaire" si besoin de passer du haut de l'entrepot vers la bas (à cause de la manière dont c'est codé..)
    if(vide_init[1] != 5 && vide_init[1] != 6 && vide_fin[1] != 5 && vide_fin[1] != 6){
        if( ((vide_init[0]==3 || vide_init[0]==4 || vide_init[0]==5 ) && (vide_fin[0]==0 || vide_fin[0]==1 || vide_fin[0]==2))
        ||  ((vide_init[0]==0 || vide_init[0]==1 || vide_init[0]==2 ) && (vide_fin[0]==3 || vide_fin[0]==4 || vide_fin[0]==5)) ) {
            // case intermédiaire correspondant au couloir vertical à gauche
            vide_fin_intermediaire[0] = 3;
            vide_fin_intermediaire[1] = 5;
            // booléen indiquant qu'il y a une case intermédiaire
            besoin_fin_intermediaire = 1;
        }
    }

    if(besoin_fin_intermediaire == 1){
//        printf("besoin_fin_intermediaire\n");
        fun_m_calcul_suite_de_pos(position_actu, vide_fin_intermediaire);
    }
    fun_m_calcul_suite_de_pos(position_actu, vide_fin);

    // déplacement correspondant à la dernière case
   // m_Nombre_deplacement = m_Nombre_deplacement + 1;

    // 'dernier déplacement'
    tab_liste_positions[m_Nombre_deplacement][0] = position_fin[0];
    tab_liste_positions[m_Nombre_deplacement][1] = position_fin[1];

//    printf("Posi fin : %d, %d\n", position_fin[0], position_fin[1]);

    if(m_Matrice_quadrillage[position_fin[0]][position_fin[1]] != 0){ // si la case d'arrivée n'est pas vide
        m_Nombre_deplacement = m_Nombre_deplacement + 1; // On fait un "dernier" déplacement pour arriver sur la case non vide
    }
}


void fun_m_calcul_suite_de_pos(int position_actu[2], int vide_fin[]){
// tant que la position finale n'est pas atteinte et check de boucle infinie
    int pos_suiv1[2], pos_suiv2[2]; // 2 possibilités pour la valeur suivante de position actu
    int delta_verti, delta_horiz; // ecart entre la position actuelle et l'arrivée

//    printf("ENTREE fun_m_calcul_suite_de_pos\nnbD : %d\n", m_Nombre_deplacement);

    while((position_actu[0] != vide_fin[0] || position_actu[1] != vide_fin[1]) && m_Nombre_deplacement<30){
// Vaut 1, 0 ou -1 selon la position relative actuelle par rapport à l'arrivée

        delta_verti  = (position_actu[0]-vide_fin[0] > 0) - (position_actu[0]-vide_fin[0] < 0); // Vaut 1 si en dessous, 0 si = , -1 si au dessus
        delta_horiz  = (position_actu[1]-vide_fin[1] > 0) - (position_actu[1]-vide_fin[1] < 0); // Vaut 1 si à droite de l'arrivée, 0 si =, -1 si à gauche
        if(delta_horiz==0){
            delta_horiz = -1;
        }
        //printf("dx dy : %d, %d\n", delta_verti, delta_horiz);

        // haut ou bas
        pos_suiv1[0] = position_actu[0] - delta_verti;
        pos_suiv1[1] = position_actu[1];
        // droite ou gauche
        pos_suiv2[0] = position_actu[0];
        pos_suiv2[1] = position_actu[1] - delta_horiz;

        //printf("Pos s1 : %d, %d\n", pos_suiv1[0], pos_suiv1[1]);
        //printf("Pos s2: %d, %d\n", pos_suiv2[0], pos_suiv2[1]);

        if(vide_fin == pos_suiv1 || vide_fin == pos_suiv2){ // la fin est adjacente
            position_actu[0] = vide_fin[0];
            position_actu[1] = vide_fin[1];
        }
        //printf("Mat S1 %d \n", m_Matrice_quadrillage[pos_suiv1[0]][pos_suiv1[1]]);
        //printf("Mat S2 %d \n", m_Matrice_quadrillage[pos_suiv2[0]][pos_suiv2[1]]);

        if(m_Matrice_quadrillage[pos_suiv1[0]][pos_suiv1[1]] ==0 && m_Matrice_quadrillage[pos_suiv2[0]][pos_suiv2[1]] ==0){ // pos_suiv1 et pos_suiv2 vides
            // cela veut dire que l'on est sur une des 2 cases carrefour
            //printf("DEUX CASES NULLES\n");
            if( (vide_fin[0] == 0 || vide_fin[0] == 5) && (vide_fin[1]==5 || vide_fin[1]==6)){
                // si l'arrivée est en (0, 5), (0, 6), (5, 5) ou (5, 6)
                // on se deplace en x
                position_actu[0] = pos_suiv1[0];
                position_actu[1] = pos_suiv1[1];
            }
            else{
                // on se deplace en y
                position_actu[0] = pos_suiv2[0];
                position_actu[1] = pos_suiv2[1];
            }
        }
        else{
            if(m_Matrice_quadrillage[pos_suiv1[0]][pos_suiv1[1]] ==0){ // pos_suiv1 non vide
                position_actu[0] = pos_suiv1[0];
                position_actu[1] = pos_suiv1[1];
                //printf("Pos actu 1 1 : %d, %d\n", position_actu[0], position_actu[1]);
            }
            else{ // pos_suiv1 vide
                if(m_Matrice_quadrillage[pos_suiv2[0]][pos_suiv2[1]] ==0){ // pos_suiv2 non vide
                    position_actu[0] = pos_suiv2[0];
                    position_actu[1] = pos_suiv2[1];
                    //printf("Pos actu 1 2 : %d, %d\n", position_actu[0], position_actu[1]);

                }
            }
        }
//        printf("Nb d : %d\n", m_Nombre_deplacement);
//        printf("Pos sauv : %d, %d\n\n", position_actu[0], position_actu[1]);
//        printf("Pos fin : %d, %d\n", vide_fin[0], vide_fin[1]);
//        printf("Pos actuu : %d, %d\n", position_actu[0], position_actu[1]);
//        printf("NB D : %d\n", m_Nombre_deplacement);
        tab_liste_positions[m_Nombre_deplacement][0] = position_actu[0];
        tab_liste_positions[m_Nombre_deplacement][1] = position_actu[1];
        m_Nombre_deplacement = m_Nombre_deplacement+1;

    }

}




void fun_m_finding_box(char box_name[2], int coord_box[2]){
    // Convertit le nom s'une case en la coordonnée correspondante
    // à partir de la matrice m_Noms_matrice_quadrillage
    // char box_name[2] : nom de la case tel qu’envoyé par l’utilisateur
	// int coord_box[2] : coordonnées de la case

    int line, col;
    int bool_char_egaux;
    // box_name contient bien un string
    if(strlen(box_name) == 0){
           // printf("Empty box\n");
    }
    else{ // box_name ne correspond pas à un corridor
        if(strncmp(box_name, " 0", 2) == 0){
           // printf("corridor\n");
        }
        else{ // boucle sur la matrice
            for(line=0; line<6; line++){
                for(col=0; col<7; col++){

    //                printf("box %s.\n", m_Noms_matrice_quadrillage[line][col]);
    //                printf("coords :%d ; %d \n", line, col);
                    bool_char_egaux = strncmp(box_name, m_Noms_matrice_quadrillage[line][col], 2) == 0;
                    if(bool_char_egaux){
                        //printf("   Bonne coord ^\n");
                        coord_box[0] = line;
                        coord_box[1] = col;
                    }
                }
            }
            //printf("box %s\n coords: %d, %d\n", box_name, coord_box[0], coord_box[1]);
        }
    }
}


void fun_m_remplir_liste_actions(int pos_init[2]){
//    -------------------------------------------------------------- EN COURS
    // Rempli m_Liste_actions avec m_Nb_actions_tot actions à partir de tab_liste_positions fournie contenant m_Nombre_deplacement positions différentes
    // boucle sur les position en utilisant la fonction fun_m_detect_deplacement_case_suiv

    m_Dir=RG_DIR;
    int pos_suiv[2]; // case suivante
    int pos_actu[2]; // case actuelle
    char m_Deplacements1case[3][1]; // action pour le changement de case
    int d, k;
    m_Nb_actions_case=0; // nombre d'actions pour le changement de case (max 3)

    m_Nb_actions_tot=0; // nombre totale d'actions pour le trajet

    pos_actu[0] = pos_init[0];
    pos_actu[1] = pos_init[1];

    for(d=0; d<m_Nombre_deplacement; d++){
        pos_suiv[0] = tab_liste_positions[d][0];
        pos_suiv[1] = tab_liste_positions[d][1];
//        printf("d: %d\npos: (%d,%d)\n", d, pos_suiv[0], pos_suiv[1]);

     //   printf("Actu : %d, %d ------ suiv : %d, %d\n", pos_actu[0], pos_actu[1], pos_suiv[0], pos_suiv[1]);
        fun_m_detect_deplacement_case_suiv(pos_actu, pos_suiv, m_Deplacements1case);
       // printf("\n\n%d ,NB ACTIONS CASE %d\n", d, m_Nb_actions_case);

        pos_actu[0] = pos_suiv[0];
        pos_actu[1] = pos_suiv[1];

        for(k=0;k<m_Nb_actions_case;k++){
            // copy du string
            strcpy(m_Liste_actions[k + m_Nb_actions_tot], m_Deplacements1case[k]);
//            m_Liste_actions[k + m_Nb_actions_tot][0] = m_Deplacements1case[k];
        }
        m_Nb_actions_tot = m_Nb_actions_tot + m_Nb_actions_case;
    }
    // Ajout de l'instruction STOP à la fin du trajet
    strcpy(m_Liste_actions[m_Nb_actions_tot], "S");
    m_Nb_actions_tot ++;
}

void fun_m_detect_deplacement_case_suiv(int pos_actu[2], int pos_suiv[2], char m_Deplacements1case[3][1]){
	// Rempli m_Liste_actions avec m_Nb_actions_tot actions à partir de tab_liste_positions fournie contenant m_Nombre_deplacement positions différentes
	// boucle sur les position en utilisant la fonction fun_m_detect_deplacement_case_suiv
    // int pos_init[2] : position de la case initiale
    // int tab_liste_positions[20][2] : liste des positions successives à parcourir par le robot

    m_Nb_actions_case = 0;
//    printf("actu x %d         suiv x %d\n", pos_actu[0], pos_suiv[0]);
//    printf("actu y %d         suiv y %d\n", pos_actu[1], pos_suiv[1]);

    if(pos_actu[0] == pos_suiv[0]){ // même ligne
      //  printf("même ligne\n");
        if(pos_suiv[1] < pos_actu[1]){ // bouger vers la gauche : Dir = 1W
          //  printf("Gauche\n");
            if(m_Dir==2){

                strcpy(m_Deplacements1case[m_Nb_actions_case], "T"); // tourner Trigo
                m_Nb_actions_case++;
            }
            if(m_Dir==3 || m_Dir==4){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire
                m_Nb_actions_case++;
            }
            if(m_Dir==3){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire (2è tour)
                m_Nb_actions_case++;
            }
            m_Dir = 1;
            strcpy(m_Deplacements1case[m_Nb_actions_case], "A"); // Avancer
            m_Nb_actions_case++;
//            printf("nb %d\n", m_Nb_actions_case);
        }
        else{ //if(pos_suiv[1] > pos_actu[1]){} // bouger vers la droite : Dir = 3E
         //   printf("Droite\n");
            if(m_Dir==2){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Trigo
                m_Nb_actions_case++;
            }
            if(m_Dir==1 || m_Dir==4){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "T"); // tourner Horaire
                m_Nb_actions_case++;
            }
            if(m_Dir==1){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire (2è tour)
                m_Nb_actions_case++;
            }
            m_Dir = 3;
            strcpy(m_Deplacements1case[m_Nb_actions_case], "A"); // Avancer
            m_Nb_actions_case++;
        }
    }
    if(pos_actu[1] == pos_suiv[1]){ // même colonne
       // printf("même colonne\n");
        if(pos_suiv[0] < pos_actu[0]){ // bouger vers le haut : Dir = 2N
         //   printf("Haut\n");
            if(m_Dir==3){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "T"); // tourner Trigo
                m_Nb_actions_case++;
            }
            if(m_Dir==1 || m_Dir==4){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire
                m_Nb_actions_case++;
            }
            if(m_Dir==4){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire (2è tour)
                m_Nb_actions_case++;
            }
            m_Dir = 2;
            strcpy(m_Deplacements1case[m_Nb_actions_case], "A"); // Avancer
            m_Nb_actions_case++;
        }
        else{ // if(pos_suiv[0] > pos_actu[0]){ // bouger vers le bas : Dir = 4S
          //  printf("Bas\n");
            if(m_Dir==1){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "T"); // tourner Trigo
                m_Nb_actions_case++;
            }
            if(m_Dir==3 || m_Dir==2){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire
                m_Nb_actions_case++;
            }
            if(m_Dir==2){
                strcpy(m_Deplacements1case[m_Nb_actions_case], "H"); // tourner Horaire (2è tour)
                m_Nb_actions_case++;
            }
            m_Dir = 4;
            strcpy(m_Deplacements1case[m_Nb_actions_case], "A"); // Avancer
            m_Nb_actions_case++;
        }
    }
  //  printf("nb ACTIONS CASE %d\n", m_Nb_actions_case);
    //printf("1 %s \n", m_Deplacements1case[0]);

}

